import 'package:aplus_chat/core/services/stream_socket.dart';
import 'package:aplus_chat/core/services/web_services.dart';
import 'package:aplus_chat/global.dart';
import 'package:aplus_chat/helper/storages.dart';
import 'package:aplus_chat/model/group_list_model.dart';
import 'package:aplus_chat/model/set_user_role_model.dart';
import 'package:aplus_chat/view/chats_list_view.dart';
import 'package:aplus_chat/view/message_view.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:socket_io_client/socket_io_client.dart';

class APlusChatMain {
  final String? secretKey;
  final String? userId;

  ///Constructor for creating CometChat object
  /// [appId] refers to your CometChat App ID.
  /// [region] The region where the app was created.
  APlusChatMain(
    this.secretKey, {
    this.userId,
  });

  /// ... INIT APlUS CHAT  (Add in you main page of project)... ///
  static Future aPlusChatInit() async {
    await AppStorages.initializeApp();
    socket.connect();
    socket.onDisconnect((_) => debugPrint('disconnect'));
    socket.io
      ..disconnect()
      ..connect();

    AppStorages.myUserId = AppStorages.getChatUserId();
    AppStorages.secKey = AppStorages.getSecKey();
    AppStorages.myUserName.value =
        AppStorages.getChatUserData(getKey: SessionKeys.chatUserName);
    AppStorages.myEmail.value =
        AppStorages.getChatUserData(getKey: SessionKeys.chatUserEmail);
    AppStorages.myFirstName.value =
        AppStorages.getChatUserData(getKey: SessionKeys.chatUserFirstName);
    AppStorages.myLastName.value =
        AppStorages.getChatUserData(getKey: SessionKeys.chatUserLastName);
    AppStorages.myProfilePicture.value =
        AppStorages.getChatUserData(getKey: SessionKeys.chatUserProfilePic);
    AppStorages.myEmail.value =
        AppStorages.getChatUserData(getKey: SessionKeys.chatUserEmail);

    AppStorages.myRoles = AppStorages.getChatUserRoles() ??
        SetUserRole(
          sendMessage: 1,
          deleteChat: 1,
          deleteMessage: 1,
          updateProfile: 1,
          editMessage: 1,
          createGroup: 1,
          createOneToOneChat: 1,
        );
  }

  /// ...  LOGIN USER ... ///
  static loginWithAPlusChat({
    required String userId,
    required String secretKey,
    required String fcmToken,
    @required Function(dynamic success)? onSuccess,
    @required Function(dynamic error)? onError,
    @required Function(dynamic connectionFail)? onConnectionFailed,
  }) async {
    Webservice.postRequest(
        context: Get.context,
        uri: Global.updateFCMToken + userId,
        body: {"secretKey": secretKey, "fcmToken": fcmToken},
        baseUrl: Global.baseApiUrl,
        hasBearer: false,
        onSuccess: (response) {
          AppStorages.setChatUserId(userId);
          AppStorages.setSecKey(secretKey);
          AppStorages.setChatUserData(fcmToken,
              setKey: SessionKeys.chatFcmToken);
          AppStorages.setChatUserData(response['item']['email'],
              setKey: SessionKeys.chatUserEmail);
          AppStorages.setChatUserData(response['item']['firstName'],
              setKey: SessionKeys.chatUserFirstName);
          AppStorages.setChatUserData(response['item']['lastName'],
              setKey: SessionKeys.chatUserLastName);
          AppStorages.setChatUserData(response['item']['username'],
              setKey: SessionKeys.chatUserName);

          AppStorages.myUserId = userId;
          AppStorages.secKey = secretKey;
          AppStorages.myEmail.value = response['item']['email'];
          AppStorages.myFirstName.value = response['item']['firstName'];
          AppStorages.myLastName.value = response['item']['lastName'];
          AppStorages.myUserName.value = response['item']['userName'];
          AppStorages.myProfilePicture.value =
              response['item']['profilePicture'];
          AppStorages.myRoles = AppStorages.getChatUserRoles();
          onSuccess!(response);
        },
        onFailure: (response) {
          onError!(response);
        },
        onConnectionFailed: (response) {
          onConnectionFailed!(response);
        });
  }

  /// ... SIGN UP (Add User) ... ///
  static signUpWithAPlusChat({
    required String userId,
    required String fcmToken,
    required String userName,
    required String secretKey,
    required String email,
    firstName,
    lastName,
    @required mobile,
    profilePicture,
    SetUserRole? setUserRole,
    @required Function(dynamic value)? onSuccess,
    @required Function(dynamic value)? onError,
    @required Function(dynamic value)? onConnectionFailed,
  }) {
    var body = {
      "userId": userId,
      "fcmToken": fcmToken,
      "username": userName,
      "firstName": firstName,
      "lastName": lastName,
      "email": email,
      "phone": mobile,
      "secretKey": secretKey,
      "profilePicture": profilePicture,
      "role": setUserRole ??
          SetUserRole(
            deleteMessage: 1,
            deleteChat: 1,
            sendMessage: 1,
            updateProfile: 1,
            editMessage: 1,
            createGroup: 1,
            createOneToOneChat: 1,
          ),
    };
    Webservice.postRequest(
        context: Get.context,
        uri: Global.createUser,
        body: body,
        baseUrl: Global.baseApiUrl,
        hasBearer: false,
        onSuccess: (response) {
          AppStorages.setChatUserRoles(setUserRole ??
              SetUserRole(
                deleteMessage: 1,
                deleteChat: 1,
                sendMessage: 1,
                updateProfile: 1,
                editMessage: 1,
                createGroup: 1,
                createOneToOneChat: 1,
              ));

          AppStorages.setChatUserData(userName,
              setKey: SessionKeys.chatUserName);
          AppStorages.setChatUserData(userId, setKey: SessionKeys.chatUserId);
          AppStorages.setChatUserData(firstName,
              setKey: SessionKeys.chatUserFirstName);
          AppStorages.setChatUserData(lastName,
              setKey: SessionKeys.chatUserLastName);
          AppStorages.setChatUserData(email, setKey: SessionKeys.chatUserEmail);
          AppStorages.setChatUserData(mobile,
              setKey: SessionKeys.chatUserMobile);
          AppStorages.setChatUserData(profilePicture,
              setKey: SessionKeys.chatUserProfilePic);
          AppStorages.setChatUserData(fcmToken,
              setKey: SessionKeys.chatFcmToken);

          if (response != null) {
            AppStorages.myUserName.value =
                AppStorages.getChatUserData(getKey: SessionKeys.chatUserName);
            AppStorages.myFirstName.value = AppStorages.getChatUserData(
                getKey: SessionKeys.chatUserFirstName);
            AppStorages.myLastName.value = AppStorages.getChatUserData(
                getKey: SessionKeys.chatUserLastName);
            AppStorages.myEmail.value =
                AppStorages.getChatUserData(getKey: SessionKeys.chatUserEmail);
            AppStorages.myMobile.value =
                AppStorages.getChatUserData(getKey: SessionKeys.chatUserMobile);
            AppStorages.myProfilePicture = AppStorages.getChatUserData(
                getKey: SessionKeys.chatUserProfilePic);
            AppStorages.chatFCMToken.value =
                AppStorages.getChatUserData(getKey: SessionKeys.chatFcmToken);
            onSuccess!(response);
          }
        },
        onFailure: (response) {
          onError!(response);
        },
        onConnectionFailed: (response) {
          onConnectionFailed!(response);
        });
  }

  /// ... LOGOUT FROM APLUS CHAT ... ///
  static logoutFromAPlusChat({
    @required Function(dynamic success)? onSuccess,
    @required Function(dynamic error)? onError,
    @required Function(dynamic onConnectionFailed)? connectionFailed,
  }) {
    Webservice.postRequest(
        context: Get.context,
        uri: Global.updateFCMToken + AppStorages.myUserId,
        body: {"secretKey": AppStorages.secKey, "fcmToken": ""},
        baseUrl: Global.baseApiUrl,
        hasBearer: false,
        onSuccess: (response) {
          if (response != null) {
            AppStorages.box.erase();
            AppStorages.myUserId = "";
            AppStorages.secKey = "";
            socket.disconnect();
            onSuccess!(response);
          }
        },
        onFailure: (response) {
          onError!(response);
        },
        onConnectionFailed: (response) {
          connectionFailed!(response);
        });
  }

  /// ... CREATE A GROUP (ONE TO ONE) ... ///
  static connectWithFriend({
    @required Function(dynamic success)? onSuccess,
    @required Function(dynamic error)? onError,
    @required Function(dynamic connectionFailed)? onConnectionFailed,
    opponentUserId,
  }) {
    var body = {
      "secretKey": AppStorages.secKey,
      "userId": AppStorages.myUserId,
      "email": AppStorages.myEmail.value,
      "members": [
        AppStorages.myUserId,
        opponentUserId,
      ],
      "isGroup": false,
      "groupImage": "",
      "name": "",
      "pinGroupForAll": 1,
      "groupPermission": [
        AppStorages.getChatUserRoles(),
      ]
    };
    Webservice.postRequest(
        context: Get.context,
        uri: Global.createGroup,
        body: body,
        baseUrl: Global.baseApiUrl,
        hasBearer: false,
        onSuccess: (response) {
          if (response != null) {
            onSuccess!(response);
          }
        },
        onFailure: (response) {
          onError!(response);
        },
        onConnectionFailed: (response) {
          onConnectionFailed!(response);
        });
  }

  /// ...  UPDATE CHAT USER PROFILE ... ///
  static updateChatUserProfile({
    @required Function(dynamic success)? onSuccess,
    @required Function(dynamic error)? onError,
    @required Function(dynamic connectionFailed)? onConnectionFailed,
    profilePic,
    firstName,
    lastName,
    userName,
    profilePicture,
  }) {
    var body = {
      "secretKey": AppStorages.secKey,
      "firstName": firstName,
      "lastName": lastName,
      "username": userName,
      "profilePicture": profilePicture,
      "userId": AppStorages.myUserId,
    };
    Webservice.postRequest(
        context: Get.context,
        uri: Global.updateProfile + AppStorages.myUserId,
        body: body,
        baseUrl: Global.baseApiUrl,
        hasBearer: false,
        onSuccess: (response) {
          if (response != null) {
            AppStorages.setChatUserData(userName,
                setKey: SessionKeys.chatUserName);
            AppStorages.setChatUserData(firstName,
                setKey: SessionKeys.chatUserFirstName);
            AppStorages.setChatUserData(lastName,
                setKey: SessionKeys.chatUserLastName);
            AppStorages.setChatUserData(profilePicture,
                setKey: SessionKeys.chatUserProfilePic);
            AppStorages.myUserName.value =
                AppStorages.getChatUserData(getKey: SessionKeys.chatUserName);
            AppStorages.myFirstName.value = AppStorages.getChatUserData(
                getKey: SessionKeys.chatUserFirstName);
            AppStorages.myLastName.value = AppStorages.getChatUserData(
                getKey: SessionKeys.chatUserLastName);
            AppStorages.myProfilePicture = AppStorages.getChatUserData(
                getKey: SessionKeys.chatUserProfilePic);
            onSuccess!(response);
          }
        },
        onFailure: (response) {
          onError!(response);
        },
        onConnectionFailed: (response) {
          onConnectionFailed!(response);
        });
  }

  /// UPDATE CHAT USER ROLES ... ///
  static updateChatUserRole({
    @required Function(dynamic success)? onSuccess,
    @required Function(dynamic error)? onError,
    @required Function(dynamic connectionFailed)? onConnectionFailed,
    SetUserRole? setUserRole,
  }) {
    var body = {
      "secretKey": AppStorages.secKey,
      "role": setUserRole,
      "userId": AppStorages.myUserId
    };
    Webservice.postRequest(
        context: Get.context,
        uri: Global.updateUserRole,
        body: body,
        baseUrl: Global.baseApiUrl,
        hasBearer: false,
        onSuccess: (response) {
          if (response != null) {
            AppStorages.setChatUserRoles(setUserRole!);
            onSuccess!(response);
          }
        },
        onFailure: (response) {
          onError!(response);
        },
        onConnectionFailed: (response) {
          onConnectionFailed!(response);
        });
  }

  /// GET ALL UNREAD COUNT ... ///
  static getCount() {
    return 50;
  }

  /// OPEN DIRECT MESSAGE VIEW OF ANOTHER USER ... ///
  static messageView({otherUserId, userId, secKey}) {
    return MessageView(
      groupListData: GroupListModel(
        imagePath: "",
        isGroup: true,
        groupId: "DfTEL5yjOzDnjwCcwqNA",
        recentMsg: "",
        groupName: "",
        latestTime: LatestTime(
          seconds: 1002,
          nanoseconds: 344,
        ),
      ),
    );
  }

  static chatListView() {
    return const ChatsListView();
  }
}
